#!/bin/sh

PROJECT=/opt/amc
PIDFILE="/tmp/amc.pid"
GUNICORN="${PROJECT}/bin/gunicorn"
CONFIG="/etc/amc/config/gunicorn_config.py"
AMCCONFIG="/etc/amc/config/amc.cfg"

inplace_edit_file(){
    TMP_FILE=`mktemp /tmp/config.XXXXXXXXXX`
    sed -e "$1" $2 > $TMP_FILE
    mv $TMP_FILE $2
}

# This is to check whether certificate file path is
# provided or no

cert_file=$(awk '/^certfile/{print $3}' $AMCCONFIG)
key_file=$(awk '/^keyfile/{print $3}' $AMCCONFIG)

cert_file_path="${cert_file%\"}"
cert_file_path="${cert_file_path#\"}"

key_file_path="${key_file%\"}"
key_file_path="${key_file_path#\"}"

if [ ! -e "$cert_file_path" ]; then
  inplace_edit_file "/^certfile/d" $CONFIG
elif [ "$(awk -F '\n' '/^certfile/{print $1}' $CONFIG)" = "" ]; then
  echo "certfile = \"<certificate_file_path>\"" >> $CONFIG
fi
if [ ! -e "$key_file_path" ]; then
  inplace_edit_file "/^keyfile/d" $CONFIG
elif [ "$(awk -F '\n' '/^keyfile/{print $1}' $CONFIG)" = "" ]; then
  echo "keyfile = \"<key_file_path>\"" >> $CONFIG
fi

cert_file="certfile = $cert_file"
key_file="keyfile = $key_file"

inplace_edit_file "s|^certfile =.*|$cert_file|g" $CONFIG
inplace_edit_file "s|^keyfile =.*|$key_file|g" $CONFIG

CMD="${GUNICORN} --daemon --config=${CONFIG} flaskapp:app"
port="8081"
get_port() {
  while read -r line           
  do              
    if echo "$line" | grep -q "bind"; then     
      data=${line##*=}
      port=${data##*:}
      port=${port%?}
    fi
  done </etc/amc/config/gunicorn_config.py
}
check_amc_status(){
    status=1
    if [ -z "`ps aux | grep amc | grep -v grep | grep gunicorn | grep master`" ]; then
        status=0
    fi
    echo $status
}

start_amc(){
  status=$(check_amc_status)
  if [ $status -eq 0 ] ; then   
      get_port  
      port_check=$(netstat -ln | grep ':$port ' | grep LISTEN)
      if [ "$port_check" != "" ]
      then
        kill -9 `ps aux | grep amc | grep -v grep | grep worker | awk '{print $2}'`
      fi
      rm -f $PIDFILE
      ${CMD}
  fi
}

echo 0 > /opt/amc/stop_signal
finish=0
while [ $finish -eq 0 ]
do
   $(start_amc)
   sleep 2
   finish=$(cat /opt/amc/stop_signal)
done
