# -*- coding: utf-8 -*-
from constants import ANSIBLE_HOSTS, ANSIBLE_TIMEOUT, ANSIBLE_TIMEOUT_ERROR
from eventlet.timeout import with_timeout
import eventlet
import sys

try:
    import ansible.runner
    from ansible.errors import AnsibleError
except ImportError as ie:
    print ie


def execute_remote_command(node_address, username, password, module_args,
background=0, module_name="command"):

    try:
        if username is "" and password is None:
            return ansible.runner.Runner(#sudo=True,
                                         pattern=node_address,
                                         module_name=module_name,
                                         module_args=module_args,
                                         background=background,
                                         transport="ssh"
                                        ).run()
        else:
            return ansible.runner.Runner(#sudo=True,
                                         pattern=node_address,
                                         module_name=module_name,
                                         module_args=module_args,
                                         background=background,
                                         remote_user=username,
                                         remote_pass=password,
                                         transport="ssh"
                                        ).run()
    except IOError:
            return dict(status="Failure", error="IO Resources unavailable")


def handle_remote_command_in_background(result, node_address, username,
password):

    if result['contacted'] != {} and 'results_file' in result['contacted'][node_address]:
        background_result = result['contacted'][node_address]['results_file']
        cmd = "cat %s" % background_result
        while True:
            result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command,
            node_address, username, password, cmd, background=0,
            timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
            if result['contacted'] != {} and result['contacted'][node_address]['stdout'] != "":
                return result
            eventlet.sleep(1)
    return None


def get_backup_progress(node_address, username, password, progress_file, namespace):

    cmd = "cat %s" % progress_file
    response = dict(status="")

    result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address,
    username, password, cmd, background=0,
    timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
    if 'error' in result:
        return result
    if result['contacted'] != {} and result['contacted'][node_address]['stdout'] != "":
        if 'failed' in result['contacted'][node_address] or result['contacted'][node_address]['stderr'] != "":
            response['status'] = "Failure"
            if 'msg' in result['contacted'][node_address]:
                response['error'] = result['contacted'][node_address]['msg']
            elif 'stderr' in result['contacted'][node_address] and result['contacted'][node_address]['stderr'] != "":
                response['error'] = result['contacted'][node_address]['stderr']
            elif 'stdout' in result['contacted'][node_address]:
                start = result['contacted'][node_address]['stdout'].find("stderr") + 10
                if start == 9:
                    response['error'] = result['contacted'][node_address]['stdout']
                else:
                    end = result['contacted'][node_address]['stdout'].find('\"', start)
                    response['error'] = result['contacted'][node_address]['stdout'][start:end]
        else:
            output = result['contacted'][node_address]['stdout']
            if output.find("command not found") != -1:
                response['status'] = "Failure"
                response['error'] = "asbackup not installed"
                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                return response
            elif output.find("could not get full path for") != -1:
                response['status'] = "Failure"
                response['error'] = "Invalid Destination Filesystem Path"
                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                return response
            elif output.find("Running out of disk space") != -1:
                response['status'] = "Failure"
                response['error'] = "Insufficient disk space"
                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                return response
            elif output.find("could not connect to host") != -1:
                response['status'] = "Failure"
                response['error'] = "Could not connect to node"
                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                return response
            elif output.find("Aerospike getmany failed") != -1:
                response['status'] = "Failure"
                response['error'] = "Backup terminated due to cluster change"
                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                return response
            elif output.find("not accessible or timed out") != -1:
                response['status'] = "Failure"
                response['error'] = "Backup failed due to cluster change"
                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                return response
            elif output.find("No objects in cluster or namespace %s doesn't exist." % namespace) != -1:
                response['status'] = "Failure"
                response['error'] = "No objects in cluster or namespace %s doesn't exist" % namespace
                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                return response
            elif output.find("Backup successfully completed.") != -1:
                response['status'] = "Success"
                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                return response
            last_pct = output.rfind("%")
            last_nonspace_before_pct = output.rfind(" ", 0, last_pct)
            if last_pct is -1 or last_nonspace_before_pct is -1:
                response['status'] = "In Progress"
                response['percentage'] = "Initiating backup"
            else:
                pct = output[last_nonspace_before_pct + 1 : last_pct + 1]
                if pct == "100%":
                    response['status'] = "Success"
                    cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                    result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                else:
                    response['status'] = "In Progress"
                    response['percentage'] = pct
    else:
        response['status'] = "Failure"
        response['error'] = "Error in tracking backup progress"
    return response


def get_restore_progress(node_address, username, password, progress_file,
aerospike_build):

    cmd = "/bin/bash -c 'ps aux | grep %s | grep -v grep'" % ("asrestore" if aerospike_build.startswith('3') else "clrestore2")
    response = dict(status="")

    result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address,
    username, password, cmd, background=0, module_name="shell",
    timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
    if 'error' in result:
        return result
    if result['contacted'] != {}:
        if 'failed' in result['contacted'][node_address] or result['contacted'][node_address]['stderr'] != "":
            response['status'] = "Failure"
            if 'msg' in result['contacted'][node_address]:
                response['error'] = result['contacted'][node_address]['msg']
            elif 'stderr' in result['contacted'][node_address] and result['contacted'][node_address]['stderr'] != "":
                response['error'] = result['contacted'][node_address]['stderr']
            elif 'stdout' in result['contacted'][node_address]:
                start = result['contacted'][node_address]['stdout'].find("stderr") + 10
                if start == 9:
                    response['error'] = result['contacted'][node_address]['stdout']
                else:
                    end = result['contacted'][node_address]['stdout'].find('\"', start)
                    response['error'] = result['contacted'][node_address]['stdout'][start:end]
        else:
            output = result['contacted'][node_address]['stdout']
            if output == "":
                cmd = "cat %s" % progress_file

                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address,
                username, password, cmd, background=0,
                timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                if 'error' in result:
                    return result
                if result['contacted'] != {} and result['contacted'][node_address]['stdout'] != "":
                    if 'failed' in result['contacted'][node_address] or result['contacted'][node_address]['stderr'] != "":
                        response['status'] = "Failure"
                        if 'msg' in result['contacted'][node_address]:
                            response['error'] = result['contacted'][node_address]['msg']
                        elif 'stderr' in result['contacted'][node_address] and result['contacted'][node_address]['stderr'] != "":
                            response['error'] = result['contacted'][node_address]['stderr']
                        elif 'stdout' in result['contacted'][node_address]:
                            start = result['contacted'][node_address]['stdout'].find("stderr") + 10
                            if start == 9:
                                response['error'] = result['contacted'][node_address]['stdout']
                            else:
                                end = result['contacted'][node_address]['stdout'].find('\"', start)
                                response['error'] = result['contacted'][node_address]['stdout'][start:end]
                    else:
                        output = result['contacted'][node_address]['stdout']
                        if output.find("command not found") != -1:
                            response['status'] = "Failure"
                            response['error'] = "asrestore not installed"
                            cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                            result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                            return response
                        elif output.find("could not get full path for") != -1:
                            response['status'] = "Failure"
                            response['error'] = "Invalid Source Filesystem Path"
                            cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                            result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                            return response
                        elif output.find("could not connect to host") != -1:
                            response['status'] = "Failure"
                            response['error'] = "Could not connect to node"
                            cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                            result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                            return response
                        elif output.find("restore failed due to unavailability of namespace in backup file: suspecting wrong data format in backup files") != -1:
                            response['status'] = "Failure"
                            response['error'] = "Namespace unavailable in backup file: suspecting wrong data format in backup files"
                            cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                            result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                            return response
                        elif output.find("restore failed due to wrong namespace name") != -1:
                            response['status'] = "Failure"
                            response['error'] = "Restore failed due to wrong namespace name"
                            cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                            result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                            return response
                        num_of_backup_files = output.count("starting restore: filename:")
                        if num_of_backup_files == 0:
                            response['status'] = "Failure"
                            response['error'] = "No backup files found"
                            cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                            result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                        else:
                            num_of_successful_restores = output.count("restore complete")
                            if num_of_backup_files == num_of_successful_restores:
                                response['status'] = "Success"
                                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
                            else:
                                response['status'] = "Failure"
                                response['error'] = "Restore unsuccessful"
                                cmd = "/bin/bash -c 'rm -rf %s'" % progress_file
                                result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background=0, module_name="shell", timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
            else:
                response['status'] = "In Progress"
    else:
        response['status'] = "Failure"
        response['error'] = "Error in tracking backup progress"
    return response


def handle_remote_command(node_address, username, password, cmd, background=0, backup_flag=False, module_name="command", available_backups=False):

    response = dict(status="")

    try:
        result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background, module_name, timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
    except AnsibleError:
        with open(ANSIBLE_HOSTS, "a") as hosts:
            hosts.write("%s\n" % node_address)
        result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background, module_name, timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))

    if 'error' in result:
        sys.stderr.write("%%%%%%%%%%%%%%%%%%%%%%%%%%\n")
        sys.stderr.write(str(result))
        return result

    if result is None or (result['contacted'] == {} and result['dark'] == {}):
        with open(ANSIBLE_HOSTS, "a") as hosts:
            hosts.write("%s\n" % node_address)

        result = with_timeout(ANSIBLE_TIMEOUT, execute_remote_command, node_address, username, password, cmd, background, module_name, timeout_value=dict(status="Failure", error=ANSIBLE_TIMEOUT_ERROR%username))
        if 'error' in result:
            sys.stderr.write("$$$$$$$$$$$$$$$$$$$$$$$$$$\n")
            sys.stderr.write(str(result))
            return result

    sys.stderr.write("@@@@@@@@@@@@@@@@@@@@@@@\n")
    sys.stderr.write(str(result))

    if background != 0 and result['contacted'] != {}:
        if backup_flag is False:
            result = handle_remote_command_in_background(result, node_address, username, password)
        else:
            if 'results_file' in result['contacted'][node_address]:
                response['status'] = "In Progress"
            else:
                response['status'] = "Failure"
                response['error'] = "Ansible background job failure"
            return response
    if result['contacted'] != {}:
        if 'failed' in result['contacted'][node_address] or result['contacted'][node_address]['stdout'].find("fail") != -1 or result['contacted'][node_address]['stderr'] != "":
            response['status'] = "Failure"
            if 'msg' in result['contacted'][node_address]:
                response['error'] = result['contacted'][node_address]['msg']
            elif 'stderr' in result['contacted'][node_address] and result['contacted'][node_address]['stderr'] != "":
                response['error'] = result['contacted'][node_address]['stderr']
            elif 'stdout' in result['contacted'][node_address]:
                start = result['contacted'][node_address]['stdout'].find("stderr") + 10
                if start == 9:
                    response['error'] = result['contacted'][node_address]['stdout']
                else:
                    end = result['contacted'][node_address]['stdout'].find('\"', start)
                    response['error'] = result['contacted'][node_address]['stdout'][start:end]
        else:
            response['status'] = "Success"
            if available_backups is True:
                from helpers import convertor
                response['available_backups'] = convertor.text_to_list(result['contacted'][node_address]['stdout'], "\n")

    elif result['dark'] != {}:
        response['status'] = "Failure"
        response['error'] = result['dark'][node_address]['msg']

    return response


def change_node_status(desired_status, node_address, username, password, aerospike_build):

    if aerospike_build.startswith('3'):
        if desired_status == "on":
            cmd = "/etc/init.d/aerospike start"
        else:
            cmd = "/etc/init.d/aerospike stop"
    else:
        if desired_status == "on":
            cmd = "/etc/init.d/citrusleaf start"
        else:
            cmd = "/etc/init.d/citrusleaf stop"

    return handle_remote_command(node_address, username, password, cmd)


def change_xdr_status(desired_status, node_address, username, password, aerospike_build):

    if aerospike_build.startswith('3'):
        if desired_status == "on":
            cmd = "/etc/init.d/aerospike_xdr resume"
        else:
            cmd = "/etc/init.d/aerospike_xdr stop"
    else:
        if desired_status == "on":
            cmd = "/etc/init.d/citrusleaf_xdr resume"
        else:
            cmd = "/etc/init.d/citrusleaf_xdr stop"

    return handle_remote_command(node_address, username, password, cmd, background=1)


